#!/usr/bin/perl -w

=begin comment info
 +-----------------------------------------------------------------------------+
 | Copyright 2001, Guardian Digital Inc.
 | $Id: cron_webtool_SystemBackup.pl,v 1.1.2.4 2001/11/02 06:59:45 rwm Exp $
 |
 | Ryan W. Maple, <ryan@guardiandigital.com>
 +-----------------------------------------------------------------------------+
=end comment info
=cut

  ###  use perl;
  use strict;
  use Getopt::Std  qw(getopts);
  use POSIX        qw(tmpnam);
  use Time::HiRes  qw(usleep ualarm gettimeofday tv_interval);


  ###  If you don't know what these are, read perlsec.
  $ENV{'PATH'}  =  "/bin:/usr/bin";
  $ENV{'ENV'}   =  "/etc/bashrc";


  ###  Get opts and perform the desired backup, or complain about usage.
  my (%opts); getopts('dwm', \%opts);

  if ($opts{'d'})    { &DoBackups('daily');   }
  elsif ($opts{'w'}) { &DoBackups('weekly');  }
  elsif ($opts{'m'}) { &DoBackups('monthly'); }
  else               { &usage();              }


  ###  Exit quitely.
  exit 0;


################################################################################
### | DoBackups()
### |
### | Iterate through all the named backups, executing ones which match $_[0].
################################################################################
sub DoBackups()
{
  my (%CONFIG);


  ###  Let's have some fun.  Display our opening TS.
  &PrintTS("", "Preparing '$_[0]' backup.\n\n");


  ###  Iterate through all our Named Backups.
  %CONFIG  =  &ReadConfig();
  foreach my $k (sort keys %CONFIG)
  {

    ###  We hit one that has the target schedule.
    if ($CONFIG{$k}->{'schedule'} eq $_[0])
    {
      my ($cmd, $ex_file, $rc);

      ###  Display start TS.
      &PrintTS("  ", "Starting backup of \"$CONFIG{$k}->{'name'}.\"\n");

        ###  Our 'base' command.
        $cmd  =  "flexbackup -fs $CONFIG{$k}->{'directory'}";

        ###  Append our -level argument.
        $cmd .=  " -level $CONFIG{$k}->{'level'}";

        ###  Write a temp file and pass as -exclude if we have an exclude list.
        if ($CONFIG{$k}->{'exclude'}) {
          $ex_file  =  POSIX::tmpnam();

          open EXCLUDEFILE, ">$ex_file";
            foreach my $l (split /\ /, $CONFIG{$k}->{'exclude'}) {
              print EXCLUDEFILE "$l\n";
            }
          close EXCLUDEFILE;

          $cmd .=  " -exclude $ex_file";
        }

        ###  Keep it quiet.
        $cmd .=  " > /dev/null 2>&1";

      ###  Finally, execute the command.
      system($cmd); $rc  =  $?;

      ###  Unlink tmp file and display ending TS.
      unlink $ex_file;
      &PrintTS("  ", "Finished backup of \"$CONFIG{$k}->{'name'}.\"\n\n");
    }
  }

  ###  All done.  Display our closing TS.
  &PrintTS("", "Finished performing '$_[0]' backup.\n");
}


################################################################################
### | ReadConfig()
### |
### | Read the WebTool's named backups into a hash of hashes.
################################################################################
sub ReadConfig()
{
  my (%C);

  open CONFIG, "/etc/webtool/SystemBackup/config";
    while (my $line = <CONFIG>)
    {
      my ($key, $value, $bname, $btype);

      ###  Yum yum.
      chomp $line;

      ###  Get out if this isn't a backupNUM_ line.
      next  if ($line !~ m/^backup[0-9]+\_/);

      ###  Slice and dice.
      ($key, $value)    =  split /\=/, $line;
      ($bname, $btype)  =  split /\_/, $key;

      ###  Set the key/value.
      $C{$bname}->{$btype} = $value;
    }

  close CONFIG;

  return %C;
}


################################################################################
### | PrintTS()
### |
### | Print a timestamp line indented by $_[0], with $_[1] as the message.
################################################################################
sub PrintTS()
{
  my ($TS); $TS  =  `date +"%b %d %T"`; chomp $TS;

  print "$_[0]", "[$TS] $_[1]";
}


################################################################################
### | usage()
### |
### | Displays the usage screen, duh.
################################################################################
sub usage()
{
  print STDERR "\nUsage:  $0 { -d | -w | -m }\n";
  print STDERR "    -d     Run all backups marked as 'daily'\n";
  print STDERR "    -w     Run all backups marked as 'weekly'\n";
  print STDERR "    -m     Run all backups marked as 'monthly'\n\n";
  exit -1;
}
